#include <LiquidCrystal.h>
#include <Wire.h>

#define ADDR_I2C_EEPROM 0xA0 >> 1  // Address du circuit 24LC256 (A0-A2 = 0)
#define DS1307_I2C_ADDRESS 0xD0 >>1 // adresse de base sur 7 bits du circuit DS1307 : $68
#define TC74_I2C_ADRESS 0x90>>1 //adresse de base sur 7 bits du circuit TC74A0 :0x90 (%1001 000) -> 0x48 (%0100 0100)
#define MAX127_I2C_ADRESS 0x50>>1      //adresse de base sur 7 bits du circuit MAX127 avec A2=A1=A0 : 0x50 (%0101 000 ) -> 0x28 (%0010 1000 )
#define I2CMotorDriverAdd 0x0F >> 1  // Address du circuit MOTORDRIVER (A0 à A3 = 1)

#define MotorSpeedSet             0x82
#define DirectionSet              0xaa
#define Nothing                   0x01

int address = 0;
String message = "";

//Température
float temperature = 0;

//Gestion du temps
byte secondes, minutes, heures, jourSemaine, jourMois, mois, annee;
byte secpass;

//Moteur
int error = 0;
byte NMLI = 0;
int Taux_MLI = 0;

//Initialisation des broches de l'afficheur : lcd(rs, rw, enable, d4, d5, d6, d7)
LiquidCrystal lcd(32, 30, 28, 26, 24, 22);

//Déclaration des fonctions
void setDateDs1307( byte ,        // 0-59
                    byte ,        // 0-59
                    byte ,          // 1-23
                    byte ,     // 1-7
                    byte ,    // 1-28/29/30/31
                    byte ,         // 1-12
                    byte );          // 0-99

void setup()
{
  byte second, minute, hour, dayOfWeek, dayOfMonth, month, year;
  int Nconv;
  int canal;

  Serial.begin(9600);      //initialise le BUS Série
  Wire.begin();            //initialise le BUS I2C
  lcd.begin(20, 4);        //initialise l'afficheur LCD

  Serial.println("Debut programme...");

  // Message présentation projet
  lcd.setCursor(0, 0);
  lcd.print("  PROJET STI2D SIN  ");
  lcd.setCursor(0, 1);
  lcd.print("- BUS I2C : FAN PC -");
  lcd.setCursor(0, 2);
  lcd.print(" Lycee JB VUILLAUME ");
  lcd.setCursor(0, 3);
  lcd.print(" ANNEE    2017/2018 ");

  //--- IMPORTANT !!! ----
  Wire.setClock(4000); // Modifie la fréquence du bus I2C pour simulation sous Proteus ISIS. A Commenter ...ou pas !!! lors du prototypage
  delay(100);

  //initialisation DS1307
  //----------------------------------------------------------------------------------------------------------------------
  Serial.println("--> INIT DS1307 ");
  getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
  delay(20);  //temporisation
  setDateDs1307(second, minute, hour, dayOfWeek, dayOfMonth, month, year);
  delay(20);  //temporisation
  getDateDs1307(&second, &minute, &hour, &dayOfWeek, &dayOfMonth, &month, &year);
  delay(20);  //temporisation
  secpass = second;

  //Initialise le circuit TC74
  //----------------------------------------------------------------------------------------------------------------------
  Serial.println("--> INIT TC74A");
  Wire.beginTransmission(TC74_I2C_ADRESS); // Début de transaction I2C
  Wire.write(0x01);
  Wire.write(0x00);  //réveil de l'horloge : Arret du mode Standby et passage en mode normal
  Wire.endTransmission();
  delay(100);
  temperature = getTempTC74();

  //initialisation MAX127 : Mode Normal + lecture de tous les Canaux
  //----------------------------------------------------------------------------------------------------------------------
  Serial.println("--> INIT MAX127");
  for (int i = 0; i < 8; i++)
  {
    canal = 0x80 | 16 * i;
    Nconv = getCAN(canal);
  }

  //Initialisation L2898 : Fixe le sens de rotation des ventilateurs :
  //----------------------------------------------------------------------------------------------------------------------

  // vérifie si MotorDriver présent  sur le bus I2C
  Serial.println("--> INIT L298");
  Wire.beginTransmission(I2CMotorDriverAdd);
  error = Wire.endTransmission();
  if (error == 0) {
    Serial.println("OK : I2CMotorDriver found...");
  } else {
    Serial.println("ERROR : I2CMotorDriver not found !");
  }
  MotorDirectionSet(0b1010);  //"0b1010" defines the output polarity, "10" means the M+ is "positive" while the M- is "negtive"



  delay(1000);
  lcd.clear();
}


void loop()
{
  byte Octet1, Octet2;
  unsigned int NTEMP;
  int Nconv = 0;
  double puissanceELEC;
  double courant, tension;

  //Lecture de la date et de l'heure
  //----------------------------------------------------------------------------------------------------------------------
  //Serial.println("--> LECTURE DS1307 ");
  getDateDs1307(&secondes, &minutes, &heures, &jourSemaine, &jourMois, &mois, &annee);

  //SI une seconde est passée ALORS
  if (secpass != secondes)
  {

    //Traitement température
    //----------------------------------------------------------------------------------------------------------------------
    Serial.println("--> LECTURE TC74A ");
    temperature = getTempTC74();
    AffichageTemp(temperature);

    // Calcul de l'octet MLI en fonction de la température
    //----------------------------------------------------------------------------------------------------------------------
    if (temperature <= 25)
    {
      NMLI = 51;
    } else {
      if (temperature >= 50) {
        NMLI = 255;
      } else {
        NMLI = (byte)((204 * (double)temperature / 25) - 135);
      }
    }

    //Traitement courant et tension
    //----------------------------------------------------------------------------------------------------------------------
    Serial.print("--> LECTURE MAX127 : ");
    Nconv = getCAN(0b10000000);                         // Conversion de la tension image du courant
    courant = (((double)Nconv / 4095) * 25  - 12.5);    //Conditionnement du courant en fonction de Nconv
    Nconv = getCAN(0b10010000);                         //Conversion de la tension d'alimentation
    tension = ((12 * (double)Nconv / 4095));            //Conditionnement de la tension en fonction de Nconv
    puissanceELEC = abs(tension) * abs(courant);        //calcul de la puissance électrique -> courant1 : 12V  - courant2 : 5V 
    AffichagePuissance(puissanceELEC, courant, tension); //affichage du résultat sur LCD
   
    //Traitement ventilo Fixe la valeur MLI sur les deux ventilateurs
    //----------------------------------------------------------------------------------------------------------------------
    MotorSpeedSetAB(NMLI, NMLI); //defines the speed of motor 1 and motor 2;
    AfficheValeurs();

    //Traitement de l'heure
    //----------------------------------------------------------------------------------------------------------------------
    AfficheHeureDate(secondes, minutes, heures, jourSemaine, jourMois, mois, annee);
    secpass = secondes;

  }
  delay(200);

}


