#include <LiquidCrystal.h>
#include <Wire.h>

#define I2CMotorDriverAdd 0x0F >> 1  // Address du circuit MOTORDRIVER (A0 à A3 = 1)

#define MotorSpeedSet             0x82
#define DirectionSet              0xaa
#define Nothing                   0x01

int error = 0;
String message = "";
float temperature = 0;
byte NMLI = 0;
int Taux_MLI = 0;

//Initialisation des broches de l'afficheur : lcd(rs, rw, enable, d4, d5, d6, d7)
LiquidCrystal lcd(32, 30, 28, 26, 24, 22);

void setup()
{
  Serial.begin(9600);      //initialise le BUS Série
  Wire.begin();            //initialise le BUS I2C
  lcd.begin(20, 4);        //initialise l'afficheur LCD

  Serial.println("Debut programme...");

  //--- IMPORTANT !!! ----
  Wire.setClock(4000); // Modifie la fréquence du bus I2C pour simulation sous Proteus ISIS. A Commenter ...ou pas !!! lors du prototypage
  delay(100);

  // vérifie si MotorDriver présent  sur le bus I2C
  Wire.beginTransmission(I2CMotorDriverAdd);
  error = Wire.endTransmission();
  if (error == 0) {
    Serial.println("OK : I2CMotorDriver found...");
  } else {
    Serial.println("ERROR : I2CMotorDriver not found !");
  }

  //Fixe le sens de rotation des ventilateurs :
  MotorDirectionSet(0b1010);  //"0b1010" defines the output polarity, "10" means the M+ is "positive" while the M- is "negtive"

  // Message présentation projet
  lcd.setCursor(0, 0);
  lcd.print("  PROJET STI2D SIN  ");
  lcd.setCursor(0, 1);
  lcd.print("- BUS I2C : MOTEUR -");
  lcd.setCursor(0, 2);
  lcd.print(" Lycee JB VUILLAUME ");
  lcd.setCursor(0, 3);
  lcd.print(" ANNEE    2017/2018 ");

  delay(1000);
  lcd.clear();
}


void loop()
{
  unsigned int NTEMP;

  //Conversion de la tension image de la Température sur A0 -> NTEMP
  NTEMP = analogRead(A0);

  //Conditionnement et Affichage de la Température
  temperature = 50 * (float)NTEMP / 1023;

  // Calcul de l'octet MLI en fonction de la température
  if (temperature <= 25)
  {
    NMLI = 51;
  } else {
    if (temperature >= 50) {
      NMLI = 255;
    } else {
      NMLI = (byte)((204*(double)temperature/25)-135);
    }
  }

  //Fixe la valeur MLI sur les deux ventilateurs
  MotorSpeedSetAB(NMLI, NMLI); //defines the speed of motor 1 and motor 2;

  
  AfficheValeurs();

  delay(1000);
}


