// *** BOUSSOLE I2C HMC6352 - Programme pour le Maître - ***
//
// par P@Eugé (pascal.euge@ac-nancy-metz.fr)
//
// Lectures de données depuis un périphérique I2C/TWI esclave : boussole HMC6352
//
// A utiliser avec le programme esclave de cet exemple.
//
// Crée le 12/08/2017
// Ce code d'exemple fait partie du domaine public.

//-------------------------------------------------------------------------------------------------------
//Librairie et déclaration des variables
//-------------------------------------------------------------------------------------------------------
#include <Wire.h>

#define MotorSpeedSet             0x82
#define PWMFrequenceSet           0x84
#define DirectionSet              0xaa
#define MotorSetA                 0xa1
#define MotorSetB                 0xa5
#define Nothing                   0x01
#define Stepernu                  0x1c

byte I2CMotorDriverAdd = 0x0F>>1; //adresse hexa de la boussole avec décalage de 1 octet pour Arduino
float azimut;
long timeMeasure = 1000;
long temps_depart;
long temps_diff;
int error = 0;

//-------------------------------------------------------------------------------------------------------
// initialisation de la carte
//-------------------------------------------------------------------------------------------------------
void setup()
{
  Wire.begin();        // joindre le bus i2c (adresse est optionnelle pour un maître)
  delay(500);
  Serial.begin(9600);  // démarré une communication série

  //--- IMPORTANT !!! ----
  Wire.setClock(4000); // Modifie la fréquence du bus I2C pour simulation sous Proteus ISIS.
  // A Commenter ...ou pas !!! lors du prototypage

  // vérifie si boussole présente  sur le bus I2C
  Wire.beginTransmission(I2CMotorDriverAdd);
  error = Wire.endTransmission();
  if (error == 0) {
    Serial.println("OK : I2CMotorDriver found...");
  } else {
    Serial.println("ERROR : I2CMotorDriver not found !");
  }



  
}

//-------------------------------------------------------------------------------------------------------
//Programme principal
//-------------------------------------------------------------------------------------------------------
void loop()
{
   // initialisation boussole (bus I2C)
  //------------------------------------------------------------------------
  if (error == 0) {
    MotorSpeedSetAB(100,100);//defines the speed of motor 1 and motor 2;
    delay(100); //this delay needed

    MotorDirectionSet(0b1010);  //"0b1010" defines the output polarity, "10" means the M+ is "positive" while the M- is "negtive"
    // make sure M+ and M- is different polatity when driving DC motors.
    delay(10000); 

    MotorDirectionSet(0b0101);  //0b0101  Rotating in the opposite direction
    delay(10000);
  }
}


void MotorSpeedSetAB(unsigned char MotorSpeedA , unsigned char MotorSpeedB)  {
  MotorSpeedA=map(MotorSpeedA,0,100,0,255);
  MotorSpeedB=map(MotorSpeedB,0,100,0,255);
  Wire.beginTransmission(I2CMotorDriverAdd); // transmit to device I2CMotorDriverAdd
  Wire.write(MotorSpeedSet);                 // set pwm header 
  Wire.write(MotorSpeedA);                   // send pwma 
  Wire.write(MotorSpeedB);                   // send pwmb    
  Wire.endTransmission();                    // stop transmitting
}

void MotorPWMFrequenceSet(unsigned char Frequence)  {    
  Wire.beginTransmission(I2CMotorDriverAdd); // transmit to device I2CMotorDriverAdd
  Wire.write(PWMFrequenceSet);               // set frequence header
  Wire.write(Frequence);                     //  send frequence 
  Wire.write(Nothing);                       //  need to send this byte as the third byte(no meaning)  
  Wire.endTransmission();                    // stop transmitting
}

void MotorDirectionSet(unsigned char Direction)  {     //  Adjust the direction of the motors 0b0000 I4 I3 I2 I1
  Wire.beginTransmission(I2CMotorDriverAdd);           // transmit to device I2CMotorDriverAdd
  Wire.write(DirectionSet);                            // Direction control header
  Wire.write(Direction);                               // send direction control information
  Wire.write(Nothing);                                 // need to send this byte as the third byte(no meaning)  
  Wire.endTransmission();                              // stop transmitting 
}


