// *** MOTOR DRIVER I2C  - Programme pour l'Esclave (DRIVER L298 I2C) - ***
//
// par P@Eugé (pascal.euge@ac-nancy-metz.fr)
//
// Commande MOTOR DC par L298 en I2C
//
// A utiliser avec le programme Maitre de cet exemple.
//
// Crée le 04/03/2018
// Ce code d'exemple fait partie du domaine public.

//-------------------------------------------------------------------------------------------------------
//Librairie et déclaration des variables
//-------------------------------------------------------------------------------------------------------
#include <Wire.h>

#define MotorSpeedSet             0x82  //Requête pour fixer la vitesse de rotation MLI
#define PWMFrequenceSet           0x84  //Requête pour fixer la fréquence de MLI 
#define DirectionSet              0xAA  //--> Requête pour fixer le SENS de rotation 0b0000 I4 I3 I2 I1
#define MotorSetA                 0xa1  //--> Requête pour fixer la vitesse de rotation MLI sur OUTA à 255
#define MotorSetB                 0xa5  //--> Requête pour fixer la vitesse de rotation MLI sur OUTB à 255
#define Nothing                   0x01  //Requête de fin de transmission (lors du changement FREQ ou DIRECTION)
#define EnableStepper             0x1a  //Requête pour activer commande moteur pas-à-pas
#define UnenableStepper           0x1b  //Requête pour désactiver commande moteur pas-à-pas
#define Stepernu                  0x1c  //Requête pour fixer la vlur de pas de rotation (commande moteur pas-à-pas)

byte I2CMotorDriverAdd = 0x0F;   // addresse I2C du MotorDriver I2C

const int pin_IN1 = 6;
const int pin_IN2 = 10;
const int pin_ENA = 9;

const int pin_IN3 = 5;
const int pin_IN4 = 11;
const int pin_ENB = 3;

int nOctet = 0;
byte Data[] = {0, 0};
byte myData[12];
byte PWMA = 0;
byte PWMB = 0;
byte direction = 0;

boolean bdebog = true;   //true pour afficher les données sur la sortie DEBOG

//-------------------------------------------------------------------------------------------------------
// initialisation de la carte
//-------------------------------------------------------------------------------------------------------
void setup()
{
  I2CMotorDriverAdd = I2CMotorDriverAdd >> 1;
  Serial.begin(9600);
  pinMode(pin_IN1, OUTPUT);
  pinMode(pin_IN2, OUTPUT);
  pinMode(pin_IN3, OUTPUT);
  pinMode(pin_IN4, OUTPUT);
  pinMode(pin_ENA, OUTPUT);
  pinMode(pin_ENB, OUTPUT);


  digitalWrite(pin_IN1, LOW);
  digitalWrite(pin_IN2, LOW);
  digitalWrite(pin_IN3, LOW);
  digitalWrite(pin_IN4, LOW);
  digitalWrite(pin_ENA, HIGH);
  digitalWrite(pin_ENB, HIGH);

  Wire.begin(I2CMotorDriverAdd);   // Joindre le bus i2c avec l'adresse 0x0F
   //--- IMPORTANT !!! ----
  Wire.setClock(4000);
  Wire.onRequest(requestEvent);    // enregistrer l'événement (lorsqu'une requête de lecture des données arrive)
  Wire.onReceive(receiveEvent);    // enregistrer l'événement (lorsqu'une réception de données arrive)
  PWMA = 0;
  PWMB = 0;
}

//-------------------------------------------------------------------------------------------------------
//Programme principal
//-------------------------------------------------------------------------------------------------------
void loop()
{
  //test traitement des commandes reçues
  if (nOctet > 0) {
    TraitementCommande();
  }
  analogWrite(pin_IN1, PWMA);
  analogWrite(pin_IN3, PWMB);

}

//-------------------------------------------------------------------------------------------------------
//Procédure de traitement des commandes envoyées par le maitre
//
// MotorSpeedSet             0x82  --> Requête pour fixer la vitesse de rotation MLI
// DirectionSet              0xaa  --> Requête pour fixer le SENS de rotation 0b0000 I4 I3 I2 I1
// MotorSetA                 0xa1  --> Requête pour fixer la vitesse de rotation MLI sur OUTA à 255
// MotorSetB                 0xa5  --> Requête pour fixer la vitesse de rotation MLI sur OUTB à 255
// Nothing                   0x01  --> Requête de fin de transmission (lors du changement FREQ ou DIRECTION)
//
//-------------------------------------------------------------------------------------------------------
void TraitementCommande()
{
  if (bdebog)
  {
    Serial.print("myData =");
    for (int i = 0; i < nOctet; i++) {
      Serial.print("  $");
      Serial.print(myData[i], HEX);
    }
    Serial.println();
  }

  //Changement vitesse de rotation
  if (myData[0] == MotorSpeedSet) {
   
    if (PWMA != myData[1]) {
      PWMA = myData[1];
    }
    if (PWMB != myData[2]) {
      PWMB = myData[2];
    }

    if (bdebog) {
      //Serial.print("MotorSpeedSet : " ); Serial.print("myData[1] = 0x" ); Serial.println(myData[1], HEX); Serial.print("myData[2] = 0x" ); Serial.println(myData[2], HEX);
      Serial.print("MotorSpeedSet : " ); Serial.print("PWMA = 0x" ); Serial.println(PWMA, HEX); Serial.print("PWMB = 0x" ); Serial.println(PWMB, HEX);
    }

  }

  //Changement sens de rotation
  if (myData[0] == DirectionSet) {
    if (bdebog) {
      Serial.print("DirectionSet : " ); Serial.print("myData[1] = 0b" ); Serial.println(myData[1], BIN);
    }
    direction = myData[1];

    //Arret Moteur
    //    digitalWrite(pin_ENA, LOW);
    //    digitalWrite(pin_ENB, LOW);

    //Fixe nouvelle direction
    digitalWrite(pin_IN1, direction & 0x01);
    digitalWrite(pin_IN2, direction & 0x02);
    digitalWrite(pin_IN3, direction & 0x04);
    digitalWrite(pin_IN4, direction & 0x08);

    //Rotation Moteurs
    //analogWrite(pin_ENA, PWMA);
    //analogWrite(pin_ENB, PWMB);
  }


  nOctet = 0;


}



//-------------------------------------------------------------------------------------------------------
// Fonction qui est exécutée lorsque des données sont demandées ( par le Maître : requestFrom)
// Cette fonction est enregistrée comme une événement ("event" en anglais), voir la fonction setup()
//-------------------------------------------------------------------------------------------------------
void requestEvent()
{
  Wire.write(Data, 2);          //Transmission des deux octets
  Wire.onRequest(requestEvent); //réactive le mode évenement sur requête
}

//-------------------------------------------------------------------------------------------------------
// Fonction qui est exécutée lorsque des données sont envoyées par le Maître.
// Cette fonction est enregistrée comme une événement ("event" en anglais), voir la fonction setup()
//-------------------------------------------------------------------------------------------------------
void receiveEvent(int byteCount)
{
  // Get and store the data.
  while (Wire.available() ) {
    myData[nOctet] = Wire.read();
    nOctet++;
  }
  Wire.onReceive(receiveEvent);
}
