#include <LiquidCrystal.h>
#include <Wire.h>

#define ADDR_I2C_EEPROM 0xA0 >> 1  // Address du circuit 24LC256 (A0-A2 = 0)

int address = 0;
String message = "";
float temperature = 0;


//Initialisation des broches de l'afficheur : lcd(rs, rw, enable, d4, d5, d6, d7)
LiquidCrystal lcd(32, 30, 28, 26, 24, 22);

void setup()
{

  Serial.begin(9600);      //initialise le BUS Série
  Wire.begin();            //initialise le BUS I2C
  lcd.begin(20, 4);        //initialise l'afficheur LCD

  Serial.println("Debut programme...");

  // Message présentation projet
  lcd.setCursor(0, 0);
  lcd.print("  PROJET STI2D SIN  ");
  lcd.setCursor(0, 1);
  lcd.print("- BUS I2C : EEPROM -");
  lcd.setCursor(0, 2);
  lcd.print(" Lycee JB VUILLAUME ");
  lcd.setCursor(0, 3);
  lcd.print(" ANNEE    2017/2018 ");

  delay(1000);
  lcd.clear();
}


void loop()
{
  byte Octet1, Octet2;
  unsigned int NTEMP;

  //Conversion de la tension image de la Température sur A0 -> NTEMP
  NTEMP = analogRead(A0);

  //Conditionnement et Affichage de la Température
  temperature = 50 * (float)NTEMP / 1023;
  lcd.setCursor(0, 1);
  lcd.print("TEMP : " + DoubleToString(temperature, 1) + " " + (char)223 + "C ");

  //Décomposition du Mot NTEMP en 2 Octets
  Octet1 = NTEMP & 0x00FF;                            //Suppression de l'Octet PF
  Octet2 = NTEMP >> 8;                                //Décalage de 8 bits vers la droite

  //Ecriture en mémoire EEPROM des deux octets
  //----------------------------------------------------------------------------------------------------------
  writeEEPROM(ADDR_I2C_EEPROM, address, Octet2);      //Ecriture de l'octet MSB
  delay(20);

  writeEEPROM(ADDR_I2C_EEPROM, address + 1, Octet1);  //Ecriture de l'octet LSB
  delay(20);

  address = address + 2;                              //on incrémente l'adresse EEPROM de +2

  //affichage des 4 dernières valeurs mémorisées une fois adress > 6 (0 -> 2 -> 4 -> 6 ... = 4 valeurs enregistrées)
  //----------------------------------------------------------------------------------------------------------
  if (address > 6) {

    Serial.println("Lecture des donnees ... ");
    message = "";

    //On boucle 4 fois pour afficher les 4 dernières valeurs
    for (int i = address - 8; i <= address - 2; i += 2) {

      //Lecture des Octets LSB et MSB
      Octet1 = readEEPROM(ADDR_I2C_EEPROM, i);
      Octet2 = readEEPROM(ADDR_I2C_EEPROM, i + 1);

      //Reconstruction du MOT NTEMP 16 bits à partir des 2 octets 8 bits
      NTEMP = Octet2 | (Octet1 << 8);

      //Conditionnement de la Température
      temperature = 50 * (float)NTEMP / 1023;

      //Préparation du message à Afficher
      if (temperature >= 10) {
        message += DoubleToString(temperature, 1) + " ";
      } else {
        message += DoubleToString(temperature, 2) + " ";
      }

    }


    //Affichage des données enregistrées sur ligne 4 du LCD
    lcd.setCursor(0, 3);
    lcd.print(message);

  }
  delay(1000);
}


